package com.tzld.piaoquan.wecom.job;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.piaoquan.wecom.common.constant.TimeConstant;
import com.tzld.piaoquan.wecom.dao.mapper.StaffMapper;
import com.tzld.piaoquan.wecom.dao.mapper.StaffWithUserMapper;
import com.tzld.piaoquan.wecom.dao.mapper.UserMapper;
import com.tzld.piaoquan.wecom.model.bo.XxlJobParam;
import com.tzld.piaoquan.wecom.model.po.*;
import com.tzld.piaoquan.wecom.service.AccessTokenService;
import com.tzld.piaoquan.wecom.utils.HttpClientUtil;
import com.tzld.piaoquan.wecom.utils.HttpPoolClient;
import com.tzld.piaoquan.wecom.utils.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.POST_WE_COM_GET_BY_USER;
import static com.tzld.piaoquan.wecom.common.constant.WeComConstant.UPDATE_USER_URL;

@Slf4j
@Component
public class WeComUserDataJob1 {

    private static final HttpPoolClient httpPoolClientDefault = HttpClientUtil.create(30000, 30000, 2000, 5000, 5, 30000);

    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private UserMapper userMapper;
    @Autowired
    private AccessTokenService accessTokenService;
    @Autowired
    private StaffWithUserMapper staffWithUserMapper;

    @XxlJob("updateStaffWithUserJob1")
    public ReturnT<String> updateStaffWithUser(String param) {
        XxlJobParam xxlJobParam = new XxlJobParam();
        if (StringUtils.isNotEmpty(param)) {
            xxlJobParam = JSONObject.parseObject(param, XxlJobParam.class);
        }
        StaffExample example = new StaffExample();
        StaffExample.Criteria criteria = example.createCriteria();
        if (xxlJobParam.getStaffId() != null) {
            criteria.andIdEqualTo(xxlJobParam.getStaffId());
        }
        List<Staff> staffList = staffMapper.selectByExample(example);
        for (Staff staff : staffList) {
            updateUserList(staff);
        }
        return ReturnT.SUCCESS;
    }


    private void updateUserList(Staff staff) {
        try {
            String cursor = "";
            do {
                String res = getUserList(staff.getCarrierId(), cursor);
                JSONObject jsonObject = JSONObject.parseObject(res);
                Integer errCode = jsonObject.getInteger("errcode");
                if (errCode != 0) {
                    log.error("updateUserList error carrierId={} cursor={}", staff.getCarrierId(), cursor);
                    return;
                }
                cursor = jsonObject.getString("next_cursor");
                JSONArray externalContactList = jsonObject.getJSONArray("external_contact_list");
                for (int i = 0; i < externalContactList.size(); i++) {
                    JSONObject externalContact = externalContactList.getJSONObject(i).getJSONObject("external_contact");
                    JSONObject followInfo =  externalContactList.getJSONObject(i).getJSONObject("follow_info");
                    Long createAt = followInfo.getLong("createtime");
                    String externalUserId = externalContact.getString("external_userid");
                    String name = externalContact.getString("name");
                    String unionId = externalContact.getString("unionid");
                    String avatar = externalContact.getString("avatar");
                    Integer type = externalContact.getInteger("type");
                    Integer gender = externalContact.getInteger("gender");
                    UserExample example = new UserExample();
                    example.createCriteria().andExternalUserIdEqualTo(externalUserId);
                    List<User> userList = userMapper.selectByExample(example);
                    User user = new User();
                    user.setExternalUserId(externalUserId);
                    user.setName(name);
                    user.setType(type);
                    user.setUnionId(unionId);
                    user.setGender(gender);
                    user.setAvatar(avatar);
                    user.setCreatedAt(createAt);
                    Long userId;
                    if (CollectionUtils.isEmpty(userList)) {
                        userMapper.insert(user);
                        userId = user.getId();
                    } else {
                        User oldUser = userList.get(0);
                        user.setId(oldUser.getId());
                        userMapper.updateByPrimaryKeySelective(user);
                        userId = oldUser.getId();
                    }
                    if (userId == null) {
                        continue;
                    }
                    StaffWithUserExample staffWithUserExample = new StaffWithUserExample();
                    staffWithUserExample.createCriteria().andStaffIdEqualTo(staff.getId()).andUserIdEqualTo(userId);
                    List<StaffWithUser> staffWithUserList = staffWithUserMapper.selectByExample(staffWithUserExample);
                    if (CollectionUtils.isEmpty(staffWithUserList)) {
                        StaffWithUser staffWithUser = new StaffWithUser();
                        staffWithUser.setStaffId(staff.getId());
                        staffWithUser.setUserId(userId);
                        staffWithUserMapper.insert(staffWithUser);
                    }
                }
            } while (StringUtils.isEmpty(cursor));
        } catch (IOException e) {
            log.error("updateUser error", e);
        }
    }

    private String getUserList(String userId, String cursor) throws IOException {
        String accessToken = accessTokenService.getWeComAccessToken();
        String url = POST_WE_COM_GET_BY_USER
                + "?access_token=" + accessToken;
        JSONObject param = new JSONObject();
        JSONArray userIdList = new JSONArray();
        userIdList.add(userId);
        param.put("userid_list", userIdList);
        param.put("limit", 100);
        if (StringUtils.isNotEmpty(cursor)) {
            param.put("cursor", cursor);
        }
        return httpPoolClientDefault.post(url, param.toJSONString());
    }


    @XxlJob("setExternalUserIdJob")
    public ReturnT<String> setExternalUserId(String param) {
        try {
            UserExample example = new UserExample();
            long count = userMapper.countByExample(example);
            int page = 1;
            int pageSize = 1000;
            long totalPageSize = count / pageSize + 1;
            for (; page <= totalPageSize; page++) {
                example.setPage(new Page<>(page, pageSize));
                List<User> userList = userMapper.selectByExample(example);
                if (CollectionUtils.isEmpty(userList)) {
                    continue;
                }
                for (User user : userList) {
                    String externalUserId3rdParty = user.getExternalUserId3rdParty();
                    String externalUserId = getExternalUserId(externalUserId3rdParty);
                    if (StringUtils.isEmpty(externalUserId)) {
                        continue;
                    }
                    User updateUser = new User();
                    updateUser.setId(user.getId());
                    updateUser.setExternalUserId(externalUserId);
                    userMapper.updateByPrimaryKeySelective(updateUser);
                }
            }
        } catch (Exception e) {
            log.error("setExternalUserId error", e);
        }

        return ReturnT.SUCCESS;
    }

    private String getExternalUserId(String externalUserId3rdParty) throws IOException {
        String weComAccessToken = accessTokenService.getWeComAccessToken();
        String url = String.format("https://qyapi.weixin.qq.com/cgi-bin/externalcontact/from_service_external_userid?access_token=%s", weComAccessToken);
        JSONObject param = new JSONObject();
        param.put("external_userid", externalUserId3rdParty);
        param.put("source_agentid", 1000009);
        String res = httpPoolClientDefault.post(url, param.toJSONString());
        JSONObject jsonObject = JSONObject.parseObject(res);
        Integer errcode = jsonObject.getInteger("errcode");
        if (errcode == 0) {
            return jsonObject.getString("external_userid");
        }
        return null;
    }


}
